<?php
/*
Plugin Name: HJI Membership
Description: Core functionality required by all other Home Junction plugins.
Version: 2.11.2
Author URI: http://www.homejunction.com
Author: Home Junction
HJI Package: hji-membership
*/

namespace hji\membership;


use \hji\common\utils\WPHelper;
use hji\membership\abstracts\Component;
use \hji\membership\controllers\CommonScripts;
use hji\membership\controllers\GForms;
use \hji\membership\controllers\ProductManager;
use \hji\membership\controllers\RequireJs;
use hji\membership\includes\PHPVersionRequirement;
use \hji\membership\models\Customer;
use \hji\common\models\JsVars;
use \hji\common\utils\Cookies;
use hji\membership\models\Settings;
use \hji\membership\models\UserSession;
use hji\membership\utils\Environment;

require_once(dirname(__FILE__) . '/autoload.php');
require_once(dirname(__FILE__) . '/third-party/Dice/dice.php');
require_once(dirname(__FILE__) . '/third-party/timber/vendor/autoload.php');
require_once(dirname(__FILE__) . '/third-party/redux-framework/ReduxCore/framework.php');


class Membership extends Component
{
    const NAME = 'HJI Membership';
    const VERSION = '2.11.2';

    const SUPPORT_EMAIL = 'help@homejunction.com';
    const SUPPORT_PHONE = '(858) 777-9533 Ext. 4';

    private static $_instance;

    private $_authHasRan = false;

    public static $file;
    public static $slug;
    public static $dir;
    public static $url;

    private $hji_mce_buttons = array();

    public $eventDispatcher;
    public $eventLogger;
    public $post_types;

    function __construct()
    {
        if (!isset($_SESSION) && !headers_sent())
        {
            session_start();
        }

        self::$file = __FILE__;
        self::$slug = basename(dirname(self::$file));
        self::$dir  = dirname(self::$file);
        self::$url  = WP_PLUGIN_URL . '/' . self::$slug;

        $this->setDiRules();

        require_once(self::$dir . '/common/utils/SiteInfo.php');
        require_once(self::$dir . '/common/models/JsVars.php');

        // Using this as action
        add_filter('tiny_mce_plugins', array($this, 'consolidateHjiMceButtons'));
        add_filter('mce_buttons_4', array($this, 'addTinyMceButtons'), 99);

        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));

        add_action('admin_print_footer_scripts', array($this, 'adminFooterScripts'));

        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('init', array($this, 'init'));

        // Registering scripts for admin and front-end at once
        // TODO: refactor script loading via correct hooks
        // TODO: eliminate theme customizer hooks usage
        add_action('init', array($this, 'register_scripts'));
        add_action('customize_controls_enqueue_scripts', array($this, 'register_scripts'));

        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
        add_action('customize_preview_init', array($this, 'enqueue_scripts'));

        add_action('activated_plugin', array($this, 'loadThisPluginFirst'), 100);

        add_action('plugins_loaded', array($this, 'registerRequireJs'), 9);

        add_action('hji_plugin_version_update' . __FILE__, array($this, 'membershipUpdate'));

        register_activation_hook(__FILE__, array($this, 'pluginActivation'));
        register_deactivation_hook(__FILE__, array($this, 'pluginDeactivation'));

        add_action('init', array($this, 'registerImageSizes'));
        add_filter('image_size_names_choose', array($this, 'addImageSizeNames'));
        add_filter('wp_mail', array($this, 'addSendGridEmailHeaders'));
        add_filter('wp_mail_from_name', array($this, 'removeWPDefaultFromName'));
        add_filter('timber/loader/paths', array($this, 'setDefaultTemplatePaths'));
        add_filter( 'http_request_host_is_external', array('\hji\membership\controllers\Downloads', 'allow_wp_catalog_host'), 10, 3 );

        rendering\twigRenderer::setActionHooks();

        $phpRequirement = new PHPVersionRequirement();
        $phpRequirement->setActionHooks();
    }


    /**
     * Instantiating classes separately only once
     *
     * @since 1.0.4
     */
    function initAccessibleClasses()
    {
        $this->settingsController   = new controllers\Settings();

        if (is_admin())
            $this->settingsController->setActionHooks();

        $this->customerModel        = Customer::getInstance();
        $this->apiController        = new controllers\APIClient($this->customerModel);
        $this->productManager       = ProductManager::getInstance();
        $this->gFormsController     = new GForms();

        if ($this->isLicensed())
        {
            if (is_admin())
            {
                $googleApiSettingsController = new controllers\GoogleApiSettings();
                $googleApiSettingsController->setActionHooks();
                $this->downloadsController = new controllers\Downloads();
            }

            $this->productManager->initActionHooks();


            // Register Membership First
            $this->productManager->registerProduct(__FILE__, 'plugin');

            // Start event logger
            $this->_startEventLogger();
        }

        $this->requireJs = RequireJs::getInstance();

        $this->gFormsController->setWPActions();
    }


    /**
     * Returns instance of the Membership Settings Model
     *
     * @since 2.6.0
     * @return \hji\membership\models\Settings  Settings Model
     */
    public static function settings()
    {
        return Settings::getInstance();
    }


    /**
     * Instance of the Membership Class
     *
     * @return Membership
     * @since   1.0.4
     */
    static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    /**
     * Checks if membership is licensed
     * based on last available authentication token.
     *
     * @return bool
     *
     * @since 1.0.4
     */
    function isLicensed()
    {
        // If authentication has ran in this runtime, token is set
        // and customer model has markets schema - we're good to go.

        if (UserSession::getToken() && $this->customerModel->markets)
        {
            return true;
        }
        else if (!$this->_authHasRan)
        {
            $authenticated = false;

            // Avoid running authentication while options being saved via options.php page
            if (!isset($_REQUEST['option_page']))
            {
                $authenticated = $this->apiController->authenticate(true);
                $this->_authHasRan = true;
            }

            return $authenticated;
        }

        return false;
    }


    /**
     * Returns array of licensed markets
     * or false
     *
     * @return array
     */
    function getMarkets()
    {
        $markets = (array) $this->customerModel->markets;
        return (!empty($markets)) ? array_keys($markets) : array();
    }


    /**
     * Forces Membership plugin to load first
     * Important, since we're calling common membership methods
     * from other HJI plugins.
     */
    function loadThisPluginFirst()
    {
        // ensure path to this file is via main wp plugin path
        $wp_path_to_this_file = preg_replace('/(.*)plugins\/(.*)$/', WP_PLUGIN_DIR."/$2", __FILE__);
        $this_plugin = plugin_basename(trim($wp_path_to_this_file));
        $active_plugins = get_option('active_plugins');
        $this_plugin_key = array_search($this_plugin, $active_plugins);

        if ($this_plugin_key)
        {
            // If it's 0 it's the first plugin already, no need to continue

            array_splice($active_plugins, $this_plugin_key, 1);
            array_unshift($active_plugins, $this_plugin);
            update_option('active_plugins', $active_plugins);
        }
    }


    function admin_menu()
    {
        add_menu_page(
            self::NAME,
            self::NAME,
            'manage_options',
            self::$slug,
            false,
            self::$url . '/resources/images/admin-menu.png'
        );
    }


    function init()
    {
        WPHelper::registerPluginVersion(__FILE__);

        add_action('wp_print_footer_scripts', '\hji\common\models\JsVars::loadVars', 0);
        add_action('admin_print_scripts', '\hji\common\models\JsVars::loadVars', 0);

        $this->di()->get('hji\\membership\\services\\SocialGraph')->init();
    }


    function registerRequireJs()
    {
        $this->requireJs->addPath('hji-membership', self::$url . '/resources/scripts/build');
        $this->requireJs->addPath('hji-membership/third-party', self::$url . '/third-party');

        $slipstreamUrl = Environment::is(Environment::DEV) ? 'https://slipstream-test-cdn.homejunction.com' :
            'https://slipstream-cdn.homejunction.com';
        $this->requireJs->addPath('slipstream', $slipstreamUrl . '/js/1.4.7', true);
        $this->requireJs->addPath('hji-common/widgets/poi', 'components/widgets/poi/poi');

        $mapboxDraw = 'https://api.mapbox.com/mapbox.js/plugins/leaflet-draw/v0.4.10/leaflet.draw';
        $this->requireJs->addPath('mapbox/draw', $mapboxDraw, true);
    }


    function register_scripts()
    {
        CommonScripts::register();

        $hjijs = Environment::is(Environment::DEV) || ((defined('SCRIPT_DEBUG') && SCRIPT_DEBUG === true)) ? 'hji.js' : 'hji.min.js';
        wp_register_script('require-js', self::$url . "/resources/scripts/build/{$hjijs}", array('jquery', 'backbone'), self::VERSION, true);

        // Do not load on themes pages (conflict with Redux Framework).
        if (!stripos($_SERVER['REQUEST_URI'], 'themes.php'))
        {
            wp_enqueue_script('require-js');
            wp_register_style('hji-membership-admin', self::$url . '/resources/styles/admin.min.css');
        }

        wp_register_style('hji-membership', self::$url . '/resources/styles/membership.min.css');

        $settings = $this->di()->get('hji\membership\models\Settings');
        $api = $this->apiController->api;

        JsVars::setVar('common', 'debug',       ((defined('SCRIPT_DEBUG') && SCRIPT_DEBUG === true)) || Environment::is(Environment::DEV));
        JsVars::setVar('common', 'licenseKey',  $settings->licenseKey);
        JsVars::setVar('common', 'areaBias',    $settings->areaBias);
        JsVars::setVar('common', 'apiVersion',  $api::$api_version);
        JsVars::setVar('common', 'siteUrl',     home_url());
        JsVars::setVar('common', 'ajaxUrl',     admin_url('admin-ajax.php'));
        JsVars::setVar('common', 'apiBaseUrl',  $api->getApiBaseUrl());
        JsVars::setVar('common', 'env',         Environment::environment());
        JsVars::setVar('common', 'cookiePath',  Cookies::COOKIEPATH);
        JsVars::setVar('common', 'cookieDomain',Cookies::COOKIE_DOMAIN);
        JsVars::setVar('common', 'customer',    $settings->customer);
        JsVars::setVar('common', 'gMapsApiKey', static::settings()->googleApi->googleApiKey);
        JsVars::setVar('common', 'hjgMapsApiKey','AIzaSyB2NnYBBeSC8UVq1OcRA5SLgfNyj1cYAd8');
        JsVars::setVar('common', 'isAdmin', is_admin());

        if (UserSession::getToken())
        {
            JsVars::setVar('common', 'token',  UserSession::getToken());
            JsVars::setVar('common', 'tokenExpires',  UserSession::getTokenExpires());
        }

        // Add markets and their centroids
        // Since we're calling customerModel before checking if customer is licensed
        // we need to check if markets data exists

        if ($this->customerModel->markets)
        {
            $markets = array();

            foreach($this->customerModel->markets as $k => $market)
            {
                $m = new \stdClass();
                $m->centroid = @$market->metadata->centroid;
                $markets[$k] = $m;
            }

            JsVars::setVar('common', 'markets', $markets);
        }
    }


    function enqueue_scripts()
    {
        if (!current_theme_supports('hji-twitter-bootstrap'))
        {
            wp_enqueue_style('hji-bootstrap');
            wp_enqueue_script('hji-bootstrap');
            wp_enqueue_style('hji-common');
        }

        if (is_admin())
        {
            wp_enqueue_script('hji-membership-admin');
            wp_enqueue_style('hji-membership-admin');
        }
        else
        {
            /**
             * Remember, you did this not because you're a dumb ass,
             * but because Slipstream is not compatible with the older versions
             * of the scripts that WP comes with. Sigh...
             */
            wp_deregister_script('underscore');
            wp_deregister_script('backbone');
            wp_register_script('underscore', 'https://cdnjs.cloudflare.com/ajax/libs/underscore.js/1.8.3/underscore-min.js', array('jquery'), true);
            wp_register_script('backbone', 'https://cdnjs.cloudflare.com/ajax/libs/backbone.js/1.2.3/backbone-min.js', array('underscore'), true);
        }
    }


    /**
     * Invoke js in the footer for common usage
     */
    function adminFooterScripts()
    {
        if (stristr($_SERVER['REQUEST_URI'], 'widgets.php'))
        {
            ?>
            <script>
                jQuery(document).ready(function(){
                    HJI.Membership.Admin.autoSaveWidgets();
                });
            </script>
        <?php
        }
    }


    /**
     * Consolidating HJI MCE buttons into a single array.
     *
     * Before TinyMCE gets initialized, we're running custom filter
     * to gather all HJI buttons, so they can be added
     * to MCE panel at once.
     *
     * @param $plugins
     * @return mixed
     */
    function consolidateHjiMceButtons($plugins)
    {
        $this->hji_mce_buttons = apply_filters('hji_mce_buttons', $this->hji_mce_buttons);

        return $plugins;
    }


    function addTinyMceButtons($buttons)
    {
        if (empty($this->hji_mce_buttons))
        {
            return $buttons;
        }

        return array_merge($this->hji_mce_buttons, $buttons);
    }


    function pluginActivation()
    {
        // deactivate Redux plugin if activated...
        $redux_slug = 'redux-framework/redux-framework.php';
        if (is_plugin_active($redux_slug))
        {
            deactivate_plugins($redux_slug);
        }

        WPHelper::registerPluginVersion(__FILE__);

        /**
         * Useful to other plugins to run their activation processes
         * like settings setup and rewrite rules flushes.
         */
        do_action('hji_membership_activation');
    }


    function pluginDeactivation(){}


    function membershipUpdate($params)
    {
        if (isset($params['plugin']) && $params['plugin'] == __FILE__)
        {
            // Force re-authentication after the update

            $this->_authHasRan = false;
            $this->apiController->authenticate(true);
        }
    }


    /**
     * @deprecated
     * @return string
     */
    public static function environment()
    {
        _deprecated_function(__FUNCTION__, '2.9.12', 'Environment::environment()');

        return Environment::environment();
    }


    /**
     * Registers default image dimensions.
     */
    function registerImageSizes()
    {
        // Used for agent headshots in email templates

        add_image_size('hji/150w150h', 150, 150, array( 'center', 'top' ));

        // Used for agent logo in email templates

        add_image_size('hji/150h', 0, 150, false);
    }


    /**
     * Adds image sizes to some selector
     *
     * Allows media manager to pass custom media sizes
     * into select image (JS) object,
     * which is used in the License Key admin UI to select the correct size.
     *
     * @param $sizes
     *
     * @return array
     */
    function addImageSizeNames($sizes )
    {
        return array_merge( $sizes, array(
            'hji/150w150h' => __( 'HJI 150x150' ),
            'hji/150h' => __( 'HJI 150 height' ),
        ) );
    }


    /**
     * Adds SendGrid API-specific header to track email activity by license key
     *
     * @param $params
     *
     * @return mixed
     */
    function addSendGridEmailHeaders($params)
    {
        $settings = new Settings();

        if ($settings->licenseKey)
        {
            $obj = new \stdClass();
            $obj->category = array($settings->licenseKey);
            $headers['X-SMTPAPI'] = json_encode($obj);

            if (isset($params['headers']))
            {
                if (is_string($params['headers']))
                {
                    $params['headers'] = trim($params['headers']) . "\r\n" . 'X-SMTPAPI: ' . json_encode($obj) . "\r\n";
                }
                else if (is_array($params['headers']))
                {
                    $params['headers'] = array_merge($params['headers'], $headers);
                }
            }
            else
            {
                $params['headers'] = $headers;
            }
        }

        return $params;
    }


    /**
     * Removes "From WordPress" from email headers.
     *
     * Implements wp_mail_from_name filter hook.
     * When wp_mail function is used directly and From Name is not set,
     * WP will use FROM: WordPress by default.
     *
     * @param $fromName string
     *
     * @return string
     */
    function removeWPDefaultFromName($fromName)
    {
        if ($fromName == 'WordPress' || !$fromName)
        {
            $settings = new Settings();
            $fromName = ($settings->customer['officeName']) ? $settings->customer['officeName'] : '';
        }

        return $fromName;
    }


    private function _startEventLogger()
    {
        $userActionLogger = $this->di()->get('hji\\membership\\models\\UserActionLogger');
        $this->eventLogger      = $this->di()->create('hji\\membership\\observers\\EventLogger', array($userActionLogger));
        $this->getServiceContainer()->getDispatcher()->attachListener($this->eventLogger);
        $this->getServiceContainer()->getDispatcher()->attachListener($this->eventLogger);
    }


    /**
     * Sets default Twig paths in the following order:
     *  Child Theme
     *  Parent Theme
     *  Plugins
     *
     * @param $paths
     *
     * @return array
     */
    function setDefaultTemplatePaths($paths)
    {
        $childThemePath  = get_stylesheet_directory();
        $parentThemePath = get_template_directory();

        if (is_child_theme())
        {
            if (false !== $path = $this->getHJITemplatesPath($childThemePath))
                $default[] = $path;

            $default[] = $childThemePath;
        }

        if (false !== $path = $this->getHJITemplatesPath($parentThemePath))
            $default[] = $path;

        $default[] = $parentThemePath;

        $paths = array_merge($default, $paths);
        $uniquePaths = array_unique($paths);

        return $uniquePaths;
    }


    protected function getHJITemplatesPath($themePath)
    {
        $hjiTemplatesDir = 'hji-templates';
        $path = trailingslashit($themePath) . $hjiTemplatesDir;

        if (file_exists($path))
            return $path;

        return false;
    }

    /**
     * @return array Post Object classes
     */
    public function getPostTypes()
    {
        return $this->post_types;
    }


    private function setDiRules()
    {
        $sharedRule = $this->di()->createRule();
        $sharedRule->shared = true;
        $this->di()->addRule(extension\postTypes\PostTypeManager::class, $sharedRule);
    }

}

Membership::getInstance();
Membership::getInstance()->initAccessibleClasses();
