<?php
namespace hji\membership\controllers;


use hji\common\models\Notices;
use hji\common\utils\Views;
use hji\membership\interfaces\Product;
use hji\membership\Membership;
use hji\membership\models\PluginEntity;
use hji\membership\models\ProductDependencies;


class Dependency
{
    /**
     * @var Notices
     */
    private $notices;

    /**
     * Products container - products added on ::dependenciesSatisfied() call
     * @var
     */
    private $products;


    function __construct()
    {
        $this->notices = new Notices();
    }


    /**
     * Action Hooks Setter
     * Has to run only once
     */
    function initActionHooks()
    {
        add_filter('extra_plugin_headers',  array($this, 'extraHeaders'));
        add_filter('extra_theme_headers',   array($this, 'extraHeaders'));

        if (is_admin())
        {
            add_action('activate_plugin',       array($this, 'checkDependenciesOnActivation'), 1, 2);
            add_action('admin_notices',         array($this, 'adminNotices'));
            add_action('admin_init',            array($this, 'setInPluginUpdateMessages'));
            add_filter('plugin_row_meta',       array($this, 'setPluginsRowMeta'), 999, 4);
        }
    }


    /**
     * Determines if product's dependencies are met
     *
     * @param Product $product
     * @return bool
     */
    public function dependenciesSatisfied(Product $product)
    {
        $this->products[] = $product;

        if ($missingDependencies = $product->dependencies->getMissingDependencies())
        {
            if ($product->getType() == 'theme')
            {
                $this->themeDependenciesNotice($missingDependencies, $product->getName());
            }

            return false;
        }

        return true;
    }


    /**
     * Registers HJI-related headers with WP-core get_file_data() parser
     * This data can be accessed via core getters of plugin and theme data.
     *
     * @param $headers
     * @return mixed
     */
    public static function extraHeaders($headers)
    {
        return array_merge($headers, ProductDependencies::getFileHeaderProperties());
    }


    /**
     * Determines if Membership plugin has an outstanding update available
     *
     * @return bool
     */
    function membershipUpdateAvailable()
    {
        $updates = get_site_transient('update_plugins');

        if (!empty($updates->response))
        {
            foreach ($updates->response as $file => $plugin)
            {
                if (isset($plugin->slug) && $plugin->slug == 'hji-membership')
                {
                    return true;
                }
            }
        }
    }


    /**
     * Renders admin notices
     */
    function adminNotices()
    {
        $this->onActivationNotice();

        $view = new Views(Membership::$dir . '/common/views/');
        echo $view->render('notices', array('notices' => $this->notices));
    }


    /**
     * Adds in-plugin update message action for each plugin
     */
    function setInPluginUpdateMessages()
    {
        $this->plugins = get_plugins();

        foreach ($this->plugins as $file => $pluginData)
        {
            add_action('in_plugin_update_message-' . $file, array($this, 'inPluginUpdateMessage'), 100, 2);
        }
    }


    /**
     * Renders upgrade notices on the wp-admin/update-core.php page
     *
     * @param $productSlug
     */
    function setUpgradeNotices($productSlug)
    {
        add_filter('puc_pre_inject_update-' . $productSlug, array($this, 'setDependencyUpgradeNotice'));
    }


    /**
     * Adds upgrade_notice property with upgrade dependencies info, to the upgrade data.
     *
     * @param $upgrade
     * @return mixed
     */
    function setDependencyUpgradeNotice($upgrade)
    {
        if (!empty($this->products))
        {
            foreach ($this->products as $product)
            {
                if ($product->getSlug() == $upgrade->slug)
                {
                    if ($missingDependencies = $product->dependencies->getMissingUpdateDependencies())
                    {
                        $upgrade->upgrade_notice = self::message('update_dependencies', self::depsToString($missingDependencies));
                        break;
                    }
                }
            }
        }

        return $upgrade;
    }


    /**
     * Renders dependencies message related to the available update
     * The message is rendered inside the table row for each respective plugin
     * inside Plugins Dashboard.
     *
     * @param $plugin_data
     * @param $r
     */
    function inPluginUpdateMessage($plugin_data, $r)
    {
//        if (!PluginEntity::isPackage($plugin_data))
//            return;

        $pluginFile = null;

        if (isset($r->plugin))
        {
            $pluginFile = $r->plugin;
        }
        else
        {
            $pluginFile = $this->getPluginFileBySlug($r->slug);
        }

        if ($pluginFile && $plugin = new PluginEntity($pluginFile))
        {
            if ($missingDependencies = $plugin->dependencies->getMissingUpdateDependencies())
            {
                echo self::message('update_dependencies', self::depsToString($missingDependencies));
            }
        }
    }


    /**
     * Renders required dependencies meta data within plugin's data row in the Plugins table
     * Hook plugin_row_meta implementation.
     *
     * @param $plugin_meta
     * @param $plugin_file
     * @param $plugin_data
     * @param $status
     * @return array
     */
    function setPluginsRowMeta($plugin_meta, $plugin_file, $plugin_data, $status)
    {
        if (PluginEntity::isPackage($plugin_data))
        {
            $plugin = new PluginEntity($plugin_file);

            $version = array_shift($plugin_meta);

            $headers = ProductDependencies::getFileHeadersHashmap();

            $meta[] = $version;

            if ($dependencies = $plugin->dependencies->getDependencies())
            {
                $meta[] = self::message('plugin_dependencies', self::depsToString($dependencies));
            }

            if ($missingDependencies = $plugin->dependencies->getMissingDependencies())
            {
                if (is_plugin_active($plugin_file))
                {
                    $plugin_meta[] = self::message('is_active_dependencies_not_met', self::depsToString($missingDependencies));
                }
                else
                {
                    $plugin_meta[] = self::message('dependencies_not_met', self::depsToString($missingDependencies));
                }
            }

            $plugin_meta = array_merge($meta, $plugin_meta);
        }

        return $plugin_meta;
    }


    /**
     * Returns plugin file path
     *
     * @param $slug
     * @return PluginEntity
     */
    function getPluginFileBySlug($slug)
    {
        $this->plugins = get_plugins();

        foreach ($this->plugins as $file => $pluginData)
        {
            if (strpos($file, $slug . '/') === 0)
            {
                return $file;
            }
        }
    }


    /**
     * Checks dependencies on plugin activation,
     * if not satisfied - sets temp transient to trigger deactivation upon page reload.
     *
     * @param $pluginFile
     * @param $network_wide
     */
    public function checkDependenciesOnActivation($pluginFile, $network_wide)
    {
        $plugin = new PluginEntity($pluginFile);

        $required = array();

        if ($plugin->getData())
        {
            if ($missingDependencies = $plugin->dependencies->getMissingDependencies())
            {
                $required[$pluginFile] = $missingDependencies;

                remove_all_actions('activate_' . $pluginFile);
                add_action('activated_plugin', array(__CLASS__, 'undo_activation_actions'), 9999, 2);
            }
        }

        if (!empty($required))
        {
            set_transient('hji_activate', $required);
        }
    }


    /**
     * Undo any activation actions which may have run for blocked activation plugins
     *
     * @param string $plugin        Name of the plugin which was blocked
     * @param bool   $network_wide  Whether the intended activation was for the network or single site
     */
    public static function undo_activation_actions($plugin, $network_wide)
    {
        remove_action(current_filter(), array(__CLASS__, __FUNCTION__));
        deactivate_plugins($plugin, $network_wide);
        do_action('deactivate_plugin', $plugin, $network_wide);
        do_action('deactivate_' . $plugin, $network_wide);
        do_action('deactivated_plugin', $plugin, $network_wide);
        $status = esc_html($_GET['plugin_status']);
        $page = esc_html($_GET['paged']);
        $s = esc_html($_GET['s']);
        wp_redirect(self_admin_url("plugins.php?plugin_status=$status&paged=$page&s=$s"));
        exit;
    }


    /**
     * Add activation failure notices to the ::$notices
     * due to dependencies being not satisfied.
     */
    function onActivationNotice()
    {
        if ($required = get_transient('hji_activate'))
        {
            delete_transient('hji_activate');

            $messages = array();
            $content = null;

            foreach ($required as $file => $dependencies)
            {
                if (!empty($dependencies))
                {
                    foreach ($dependencies as $dependency)
                    {
                        $messages[] = sprintf('%s v. %s', $dependency->name, $dependency->version);
                    }

                    $plugin = new PluginEntity($file);
                    $data = $plugin->getData();

                    $content .= $data['Name'] . ' requires ' . implode(', ', $messages);
                }
            }

            if ($content)
            {
                $content = self::message('on_activate_dependencies_not_met', $content);
            }

            $this->notices->addNotice($content, 2);
        }
    }


    /**
     * Adds theme dependency notice to the ::$notices container.
     *
     * @param array $deps dependencies
     * @param       $productName
     */
    function themeDependenciesNotice(array $deps, $productName)
    {
        $error = self::message('theme_dependencies_not_met', self::depsToString($deps), $productName);
        $this->notices->addNotice($error, 2);
    }


    /**
     * Stringifies array of dependency entities
     *
     * @param array $deps
     * @return null|string
     */
    protected static function depsToString(array $deps)
    {
        $result = null;

        if (!empty($deps))
        {
            $strings = array();

            foreach ($deps as $dependency)
            {
                $name = ($dependency->type == 'theme') ? self::_getThemeName($dependency->name) : $dependency->name;

                $strings[] = sprintf('%s v. %s', $name, $dependency->version);
            }

            $last   = array_pop($strings);
            $n      = count($strings);

            if ($n == 0)
            {
                $result .= $last;
            }
            else
            {
                $result .= implode(', ', $strings) . ' and ' . $last;
            }
        }

        return $result;
    }


    protected static function _getThemeName($template)
    {
        $theme = wp_get_theme($template);
        return $theme->get('Name');
    }


    /**
     * Returns message string based on code
     *
     * @param      $code
     * @param      $dependenciesStr
     * @param null $productName
     * @return string
     */
    protected static function message($code, $dependenciesStr, $productName = null)
    {
        $megaphone = '<i class="dashicons dashicons-megaphone"></i> ';
        $error     = '<i class="dashicons dashicons-dismiss"></i> ';
        $info      = '<i class="dashicons dashicons-info"></i> ';

        $theme = ($productName) ? $productName . ' theme' : 'Theme';

        $messages = array(

            'update_dependencies' => sprintf('<div class="hji-required-dependencies"><span style="color: #d54e21">%s<strong>%s</strong> </span> %s</div>', $megaphone, 'Update Requires:', $dependenciesStr),

            'is_active_dependencies_not_met' => sprintf('<div class="hji-required-dependencies"><span style="color: red">%s<strong>%s</strong> </span> %s</div>', $error, 'Plugin is inactive. Dependencies are not met:', $dependenciesStr),

            'dependencies_not_met' => sprintf('<div class="hji-required-dependencies"><span style="color: #d54e21">%s<strong>%s</strong> </span> %s</div>', $info, 'Dependencies not met:', $dependenciesStr),

            'theme_dependencies_not_met' => sprintf('<span style="color: #dd3d36">%s<strong>%s</strong> </span> %s', $error, $theme . ' dependencies are not met. Please update the following items:', $dependenciesStr),

            'on_activate_dependencies_not_met' => sprintf('<div class="hji-required-dependencies"><span style="color: #d54e21">%s<strong>%s</strong> </span> %s</div>', $error, 'Plugin activation failed. Dependencies were not met:', $dependenciesStr),

            'plugin_dependencies' => sprintf('<span class="hji-required-dependencies">Dependencies: %s<span>', $dependenciesStr),

        );

        return isset($messages[$code]) ? $messages[$code] : '';
    }


    /**
     * Returns array of items sorted in the order satisfying dependencies
     *
     * @param $dependenciesArray array(item => array(dep1, dep2..)...);
     * @return array
     */
    public function resolveDependencies($dependenciesArray)
    {
        $dependencyResolver = new DependencyResolver();

        foreach ($dependenciesArray as $item => $dependencies)
        {
            $dependencyResolver->add($item, $dependencies);
        }

        return $dependencyResolver->getResolved();
    }
}


class DependencyResolver
{
    protected $deps;


    function __construct()
    {
        $this->deps = new \WP_Dependencies();
    }


    function add($item, $dependencies)
    {
        $this->deps->add($item, false, $dependencies);
        $this->deps->enqueue($item);
    }


    function getResolved()
    {
        return $this->deps->do_items();
    }
}