<?php
namespace hji\membership\controllers;

use hji\common\Plugin;
use hji\common\utils\Dependencies;
use hji\membership\interfaces\Product;
use hji\membership\Membership;


/**
 * Class ProductManager
 *
 * - Allows plugins and themes to register themselves with Membership
 * - Initializes registered plugins
 * - Schedules automatic updates for registered plugins and themes
 *
 * @since 2.0
 * @package hji\membership\controllers
 */
class ProductManager
{
    private static $_instance;

    /**
     * Product Registry Object
     * @var ProductRegistrar
     */
    protected $productRegistrar;

    protected $dependencyController;

    protected $updateManager;

    /**
     * Collection of registry entries to init (their dependencies are satisfied)
     * @var array
     */
    protected $toInit;


    private function __construct()
    {
        $this->productRegistrar     = ProductRegistrar::getInstance();
        $this->dependencyController = new Dependency();

        // ensures only admin and wp-cli users can update things
        if ($this->_currentUserCanUpdateProducts())
        {
            $this->updateManager        = UpdateManager::getInstance();
        }
    }


    static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    /**
     * Sets up action hooks
     *
     * @codeCoverageIgnore
     */
    function initActionHooks()
    {
        add_action('plugins_loaded',    array($this, 'registerMembershipPlugins'));
        add_action('plugins_loaded',    array($this, 'initMembershipPlugins'));
        add_action('after_setup_theme', array($this, 'registerMembershipThemes'));
        add_action('after_setup_theme', array($this, 'initMembershipThemes'));

        $this->dependencyController->initActionHooks();
    }


    /**
     * Registers HJI Plugins
     */
    function registerMembershipPlugins()
    {
        $products = apply_filters('hji_membership_register_plugin', array());

        $products = array_filter($products);

        if (!empty($products))
        {
            $this->_registerProducts($products, 'plugin');
        }
    }


    /**
     * Initializes registered products
     * Fires up product callbacks
     * Schedules product updates
     *
     * @param string $productType
     */
    function initMembershipProducts($productType)
    {
        if ($registryEntries = $this->getRegistryEntriesToInit())
        {
            foreach ($registryEntries as $registryEntry)
            {
                $currentType = $registryEntry->product->getType();

                if ( $currentType == $productType)
                {
                    // Initialize product

                    $this->_initProduct($registryEntry);

                    // Schedule Product Updates
                    // - Membership plugin & themes always get a pass
                    // - The rest of the products only get updates if Membership is up to date.

                    if ($this->updateManager)
                    {
                        if (($registryEntry->product->getSlug() == 'hji-membership' ||
                                $registryEntry->product->getType() == 'theme') ||
                            !$this->dependencyController->membershipUpdateAvailable())
                        {
                            $this->_scheduleProductUpdates($registryEntry->product);
                        }
                    }
                }
            }
        }
    }


    function initMembershipThemes()
    {
        $this->initMembershipProducts('theme');
    }


    function initMembershipPlugins()
    {
        $this->initMembershipProducts('plugin');

        do_action('hji_membership_init');
    }


    /**
     * Registers HJI Themes
     */
    function registerMembershipThemes()
    {
        $products = apply_filters('hji_membership_register_theme', array());

        if (!empty($products))
        {
            $this->_registerProducts($products, 'theme');
        }
    }


    /**
     * Registers array of same-type products
     *
     * @param $products
     * @param $type
     *
     * @throws \Exception
     */
    protected function _registerProducts($products, $type)
    {
        $productProperties = null;

        foreach ($products as $product)
        {
            if ($plugin = $this->_isPluginInstance($product))
            {
                $productProperties = [
                    'file'      => $plugin->file,
                    'callback'  => [$plugin, 'init']
                ];
            }
            else if (is_array($product))
            {
                $productProperties = $this->_validateProductProperties($product);
            }

            if ($productProperties)
            {
                $this->registerProduct($productProperties['file'], $type, $productProperties['callback']);
            }
        }
    }


    /**
     * @param $object
     *
     * @return null|object
     * @throws \Exception
     */
    protected function _isPluginInstance($object)
    {
        $plugin = null;

        if (is_array($object))
        {
            if (isset($object[0]) && isset($object[1]))
            {
                if (is_string($object[0]) && is_array($object[1]))
                {
                    $plugin = Membership::getInstance()->di()->create($object[0], $object[1]);
                }
            }
        }
        else if (is_object($object))
        {
            $plugin = $object;
        }

        if ($plugin instanceof Plugin && $plugin->file)
            return $plugin;
    }


    /**
     * Adds product to the registry
     *
     * @param      $file
     * @param      $type
     * @param null $callback
     */
    function registerProduct($file, $type, $callback = null)
    {
        // Add to the registry

        $this->productRegistrar->register($file, $type, $callback);

        $registryEntry = $this->productRegistrar->getEntry($file);

        if ($this->dependencyController->dependenciesSatisfied($registryEntry->product))
        {
            $this->toInit[$registryEntry->product->getName()] = $registryEntry;
        }
    }


    /**
     * Validates properties of the product that is being registered
     *
     * @param $product
     * @return array|bool
     */
    protected function _validateProductProperties(array $product)
    {
        // Properties
        $required   = array('file');
        $default    = array(
            'file'      => null,
            'callback'  => null
        );

        $validated = array();

        foreach ($default as $property => $value)
        {
            if (isset($product[$property]))
            {
                $validated[$property] = $product[$property];
            }
            else
            {
                $validated[$property] = $default[$property];
            }

            if (in_array($property, $required) && empty($validated[$property]))
            {
                return false;
            }
        }

        return $validated;
    }


    /**
     * Fires up product callbacks if such exist
     *
     * @param RegistryEntry $registryEntry
     */
    protected function _initProduct(RegistryEntry $registryEntry)
    {

        if (is_callable($registryEntry->callback))
        {
            add_action('hji_membership_init', $registryEntry->callback);

            // Set status
            $registryEntry->setStatus(1);
        }
    }


    /**
     * Schedules product updates
     *
     * @param Product $product
     */
    protected function _scheduleProductUpdates(Product $product)
    {
        $this->updateManager->scheduleUpdates($product);
        $this->dependencyController->setUpgradeNotices($product->getSlug());
    }


    /**
     * Returns Registry Entries that are sorted in resolved dependencies order
     *
     * @return array of RegistryEntry objects
     */
    function getRegistryEntriesToInit()
    {
        if (!empty($this->toInit))
        {
            // Prepare dependencies array - array(item => array(dep1, dep2..)...)
            $dependenciesArray = array();

            foreach ($this->toInit as $registryEntry)
            {
                $dependenciesArray[$registryEntry->product->getName()] = $this->_getDependencyNames($registryEntry->product);
            }

            // Resolve dependencies - sorts items in the correct order
            if ($resolved = $this->dependencyController->resolveDependencies($dependenciesArray))
            {
                $result = array();

                foreach ($resolved as $item)
                {
                    $result[$item] = $this->toInit[$item];
                }

                return $result;
            }
        }
    }


    protected function _getDependencyNames(Product $product)
    {
        $dependencyNames = array();

        if ($dependencies = $product->dependencies->getDependencies())
        {
            foreach ($dependencies as $dependency)
            {
                $dependencyNames[] = $dependency->name;
            }
        }

        return $dependencyNames;
    }


    /**
     * Check if updates should be checked/run
     *
     * Checks to see if the user is an admin who can update plugins / themes or using WP-CLI
     *
     * @return bool
     */
    protected function _currentUserCanUpdateProducts()
    {
        return (is_admin() || (defined('WP_CLI') && WP_CLI)) ? true : false;
    }
}