<?php

namespace hji\membership\controllers;

use hji\membership\includes\CustomerInfoSynchronizer;
use \hji\membership\Membership;

use \hji\common\utils\HTTP;
use \hji\common\utils\StringUtil;
use \hji\common\utils\Validator;
use \hji\common\utils\View;

use \hji\common\models\Notices;

use hji\membership\models\Customer;
use \hji\membership\models\Settings as M_Settings;


class Settings
{
    private $notices;



    public function __construct()
    {
        $this->notices = new Notices();
        $this->customerSynchronizer = new CustomerInfoSynchronizer();
    }


    public function setActionHooks()
    {
        // Global Admin Hooks that need to run everywhere in the Admin
        add_action('admin_menu',    array($this, 'admin_menu'));
        add_action('admin_menu',    array($this, 'addCRMLink'));
        add_action('admin_init',    array($this, 'registerSettings'));

        add_action('admin_notices', array($this, 'admin_notices'));

        // Processing posted settings right before Membership instantiates
        // hji_membership_init hook - before all HJI plugins loaded.
        // This way if license is invalid, they will not show up in the admin after
        // posted data is processed.

        add_filter('pre_update_option_' . M_Settings::KEY, array($this, 'preOptionsUpdate'), 10, 2);
        add_action('update_option_hji-membership-settings', array($this, 'onOptionsUpdated'), 10, 2);
    }


    /**
     * Sets Page-Specific Action Hooks
     * which will executed only on the given page
     *
     * @param string $page Settings Page ID defined by WP
     */
    protected function setPageHooks($page)
    {
        add_action('admin_print_styles-' . $page, array($this, 'admin_scripts'));
        add_action('load-' . $page, [$this->customerSynchronizer, 'autosyncCustomerInfo']);
        add_action('load-' . $page, [$this, 'setOptionsUpdatedNotice']);
    }


    /**
     * Removes customer information from the Options if License Key has changed.
     *
     * This action will force Membership to fetch customer info during next authentication.
     *
     * @param $newValue
     * @param $oldValue
     *
     * @return mixed
     */
    public function preOptionsUpdate($newValue, $oldValue)
    {
        // Make sure this applies only to options submitted via Membership settings form.

        if (!$this->isUpdating())
            return $newValue;

        if ($this->licenseKeyChanged($newValue, $oldValue))
            $newValue = $this->flushCustomerInfo($newValue);

        return $newValue;
    }


    /**
     * Re-authenticates upon options update
     * and executes settings-updated Membership hook.
     */
    public function onOptionsUpdated()
    {
        if (!$this->isUpdating())
            return;

        // Re-authenticate
        // Instantiating new objects to avoid previously constructed singletons
        $apiClientController = new APIClient(new Customer());
        $apiClientController->authenticate();

        // Sync customer data with Keymaster
        $this->customerSynchronizer->syncCustomerInfo();
    }


    public function registerSettings()
    {
        register_setting(M_Settings::KEY, M_Settings::KEY);
    }


    public function admin_menu()
    {
        $page = add_submenu_page(Membership::$slug,
                         Membership::NAME . ' | Settings',
                         'License Key',
                         'manage_options',
                         Membership::$slug,
                         [$this, 'renderSettingsPage']
        );

        $this->setPageHooks($page);
    }


    /**
     * Adds HJI(IRE) CRM external link to the Admin Menu
     *
     * A dirty hack since we're modifying global $menu array directly.
     */
    public function addCRMLink()
    {
        global $menu;

        $settings = new M_Settings();

        if (!Membership::getInstance()->isLicensed() || !$settings->crm)
            return;

        $url = "http://{$settings->crm['url']}/admin";

        $menu[] = array(
            'HJI CRM',
            apply_filters('hji-membership/admin/crm_menu_access_capability', 'manage_options'),
            $url,
            'HJI CRM',
            'menu-top hji-crm-link',
            '',
            Membership::$url . '/resources/images/admin-menu.png'
        );
    }


    public function admin_scripts()
    {
        wp_enqueue_media();
        wp_enqueue_script('hji-membership-admin');
        wp_enqueue_style('hji-membership-admin');
    }


    /**
     * Renders admin notices related to Membership Settings
     */
    public function admin_notices()
    {
        $settings = new M_Settings();
        $page = strtolower(HTTP::getParameter('page'));

        if ($page != Membership::$slug)
        {
            if (Validator::isLicenseKey($settings->licenseKey) === false)
            {
?>
                <div class='updated'>
                    <p>
                        Before you can start using your <strong>Home Junction</strong> plugins, you must
                        <a href='<?php echo admin_url('admin.php?page=' . Membership::$slug) ?>'>enter</a>
                        a valid license key.
                    </p>
                </div>
<?php
            }
        }

        if (Membership::getInstance()->isLicensed())
        {
            if ((!isset($settings->customer['photo']) || empty($settings->customer['photo'])) ||
            (!isset($settings->customer['logo']) || empty($settings->customer['logo'])))
            {
                // TODO: make notice dismissable via is-dismissible class + custom handler.
                // Keep in mind, that the dismissing has to be handled by us in order to make it persist
                // At some point WP wants to handle dismissions in the core (last paragraph):
                // https://make.wordpress.org/core/2015/04/23/spinners-and-dismissible-admin-notices-in-4-2/
                ?>
                <div class="notice notice-warning">
                    <p>Please update your information including Photo and Logo to be used in your Home Junction Marketing Campaigns. <a href="<?php echo admin_url('admin.php?page=' . Membership::$slug) ?>">Click here to update</a>.</p>
                </div>
                <?php
            }
        }
    }


    /**
     * Renders settings page HTML
     */
    public function renderSettingsPage()
    {
        $customer   = new Customer();
        $markets    = $customer->markets;
        $isLicensed = Membership::getInstance()->isLicensed();
        $settings   = new M_Settings();

        echo View::render(Membership::$dir . '/classes/views/settings.phtml',
                          array(
                              'settings' => $settings,
                              'notices' => $this->notices,
                              'customer' => $customer,
                              'markets' => $markets,
                              'isLicensed' => $isLicensed)
        );
    }


    /**
     * Executes action hook "settings-updated" after WP redirects
     * from options.php (upon form submission) back to the Membership settings page.
     *
     * This is important, since Membership re-authenticates and resets Customer Model
     * upon redirection.
     *
     * IMPORTANT: Making API calls before redirect, can lead to unexpected results, due
     * to the fact that customer model still uses previous licensee information.
     */
    public function setOptionsUpdatedNotice()
    {
        if (!is_admin())
        {
            return;
        }

        $page = strtolower(HTTP::getParameter('page'));

        if ($page == Membership::$slug)
        {
            $updated = HTTP::getParameter('settings-updated');

            if (StringUtil::toBoolean($updated) == true)
            {
                $this->notices->addNotice('Your settings have been saved.');
            }
        }
    }


    /**
     * Determines if state of the Membership options is being currently updated
     * via Settings form submission
     *
     * @return bool
     */
    public function isUpdating()
    {
        return (isset($_POST['option_page']) && isset($_POST[M_Settings::KEY]));
    }


    /**
     * Determines if license key was changed on Options Update
     *
     * @param $newOptions
     * @param $oldOptions
     *
     * @return bool
     */
    protected function licenseKeyChanged($newOptions, $oldOptions)
    {
        $newLK = (isset($newOptions['license-key'])) ? $newOptions['license-key'] : null;
        $oldLK = (isset($oldOptions['license-key'])) ? $oldOptions['license-key'] : null;

        return ($newLK == $oldLK) ? false : true;
    }


    /**
     * Removes customer information form given options
     *
     * @param $options
     *
     * @return mixed
     */
    protected function flushCustomerInfo($options)
    {
        if (isset($options['customer']))
            unset($options['customer']);

        return $options;
    }
}
