<?php

namespace hji\membership\extension\postTypes;


use hji\membership\Membership;
use hji\membership\thirdparty\doctrine\inflector\Inflector;


final class PostTypeDefinition
{
    public $name;
    public $singular_name;
    public $plural_name;
    public $public = true;
    public $show_in_rest = false;

    public $enter_title_here;
    public $featured_image;

    protected $fieldPrefix = '_hj_';
    protected $fieldsSchema = [];
    protected $taxonomiesSchema =  [];
    protected $adminColsSchema = [];
    protected $adminFiltersSchema = [];

    protected $textdomain = 'hji-texdomain';


    public function __construct($post_type, array $args = [])
    {
        $this->name = sanitize_key($post_type);
        $this->setDefaults();
        $this->setProperties($args);
    }


    public function fieldPrefix()
    {
        return $this->fieldPrefix;
    }


    public function fieldsSchema()
    {
        return $this->fieldsSchema;
    }


    public function taxonomiesSchema()
    {
        return $this->taxonomiesSchema;
    }


    public function adminColsSchema()
    {
        return $this->adminColsSchema;
    }


    public function adminFiltersSchema()
    {
        return $this->adminFiltersSchema;
    }


    public function textdomain()
    {
        return $this->textdomain;
    }


    protected function setDefaults()
    {
        /**
         * @see  WP_Post_Type
         * @link https://codex.wordpress.org/Function_Reference/register_post_type#Arguments
         */
        $defaultProperties = [
            'labels'                => [],
            'description'           => '',
            'public'                => true,
            'hierarchical'          => false,
            'exclude_from_search'   => null,
            'publicly_queryable'    => null,
            'show_ui'               => null,
            'show_in_menu'          => null,
            'show_in_nav_menus'     => null,
            'show_in_admin_bar'     => null,
            'menu_position'         => 6,
            'menu_icon'             => Membership::$url . '/resources/images/admin-menu.png',
            'capability_type'       => 'post',
            'capabilities'          => [],
            'map_meta_cap'          => null,
            'supports'              => [
                'title',
                'editor',
                'thumbnail'
            ],
            'register_meta_box_cb'  => null,
            'taxonomies'            => [], // registered taxonomies (core) like categories, tags
            'has_archive'           => true,
            'rewrite'               => true,
            'query_var'             => true,
            'can_export'            => true,
            'delete_with_user'      => null,
            'show_in_rest'          => false,
            'rest_base'             => null,
            'rest_controller_class' => false,
        ];

        $extendedProperties = [
            'singular'       => null,
            'plural'         => null,
            'site_filters'   => null,  # Custom arg
            'site_sortables' => null,  # Custom arg
            'show_in_feed'   => false, # Custom arg
            'archive'        => null,  # Custom arg
            'featured_image' => null,  # Custom arg
        ];


        $adminProperties = [
            'quick_edit'       => true,  # Custom arg
            'dashboard_glance' => true,  # Custom arg
            'admin_cols'       => null,  # Custom arg
            'admin_filters'    => null,  # Custom arg
            'enter_title_here' => null,  # Custom arg
        ];

        $defaults = array_merge($defaultProperties, $extendedProperties, $adminProperties);

        // Only set values that are not null,
        // let the rest of the params to be handled
        // by Extended CPTs and WP_Post_Type further down the road

        foreach ($defaults as $property => $value)
        {
            if (is_bool($value) || !empty($value))
                $this->$property = $value;
        }
    }


    protected function setProperties($args)
    {
        $args = wp_parse_args( $args );

        // Presets
        $presets = [
            'slug'     => Inflector::pluralize($this->name),
            'singular' => $args['singular_name'] ?? $this->name,
            'plural'   => $args['plural_name'] ?? Inflector::pluralize($this->name)
        ];

        $presets['labels'] = $this->defineLabels(
            $presets['singular'], $presets['plural'], $this->textdomain);

        # This allows the 'labels' and 'rewrite' args to contain all, some, or no values:
        foreach ( [ 'labels', 'rewrite' ] as $arg ) {
            if ( isset( $args[ $arg ] ) && is_array( $args[ $arg ] ) ) {
                $presets[ $arg ] = array_merge( ($presets[ $arg ] ?? []), $args[ $arg ] );
                unset($args[ $arg ]);
            }
        }

        $params = array_merge($presets, $args);

        // unset post_type property, since it will conflict with WP_Post_Type
        if (isset($params['post_type']))
            unset($params['post_type']);

        if (isset($params['singular_name']))
            unset($params['singular_name']);

        if (isset($params['plural_name']))
            unset($params['plural_name']);

        foreach ($params as $property => $value)
        {
            $this->$property = $value;
        }

        if ($this->show_in_rest === true && !isset($this->rest_base))
            $this->rest_base = $this->slug;
    }


    protected function defineLabels($singular, $plural, $textdomain = null)
    {
        $singular = ucfirst($singular);
        $plural = ucfirst($plural);
        return [
            'name'                  => _x($plural, 'Post type general name', $textdomain),
            'singular_name'         => _x($singular, 'Post type singular name', $textdomain),
            'menu_name'             => _x($plural, 'Admin Menu text', $textdomain),
            'name_admin_bar'        => _x($singular, 'Add New on Toolbar', $textdomain),
            'add_new_item'          => __('Add New ' . $singular, $textdomain),
            'edit_item'             => __('Edit ' . $singular, $textdomain),
            'new_item'              => __('New ' . $singular, $textdomain),
            'view_item'             => __('View ' . $singular, $textdomain),
            'view_items'            => __('View ' . $plural, $textdomain),
            'search_items'          => __('Search ' . $plural, $textdomain),
            'not_found'             => __('No ' . strtolower($plural) . ' found.', $textdomain),
            'not_found_in_trash'    => __('No ' . strtolower($plural) . ' found in Trash.', $textdomain),
            'parent_item_colon'     => __('Parent ' . $plural . ':', $textdomain),
            'all_items'             => __('All ' . $plural, $textdomain),
            'archives'              => __($singular . ' Archives', $textdomain),
            'attributes'            => __($singular . ' Attributes', $textdomain),
            'insert_into_item'      => __('Insert into ' . strtolower($singular), $textdomain),
            'uploaded_to_this_item' => __('Uploaded to this ' . strtolower($singular), $textdomain),
            'filter_items_list'     => __('Filter ' . strtolower($plural) . ' list', $textdomain),
            'items_list_navigation' => __($plural . ' list navigation', $textdomain),
            'items_list'            => __($plural . ' list', $textdomain)
        ];
    }
}