<?php

namespace hji\membership\models;

/**
 * Class UserSession
 *
 * @since 2.0
 * @package hji\membership\models
 */

class UserSession
{
    private static $_instance;

    protected $token = null;

    protected $tokenExpires = null;

    protected $user = null;

    protected $vars = array();


    private function __construct()
    {
        if (!isset($_SESSION))
        {
            session_start();
        }

        /**
         * IMPORTANT: do not remove!
         * When session already exists, we only call getToken() to check if it's valid
         * but instance properties get never populated from the $_SESSION
         * When calling setUser() => _updateSession() we overwrite $_SESSION with null properties
         * form the current class instance. That's why instance property values need to be
         * pre-populated from $_SESSION inside the constructor.
         */
        $this->token        = self::getToken();
        $this->tokenExpires = self::getTokenExpires();
        $this->user         = self::getUser();
        $this->vars         = self::getVars();
    }


    public static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    public static function getSessionId()
    {
        return 'hjiUserSession_BlogId_' . get_current_blog_id();
    }


    /**
     * Sets user session with API token and token expiration
     *
     * @param $token
     * @param $tokenExpires
     */
    public static function start($token, $tokenExpires)
    {
        // delete existing session
        self::_deleteSession();

        $instance = self::getInstance();

        $instance->token        = $token;
        $instance->tokenExpires = $tokenExpires;
        $instance->user         = null;

        self::_updateSession();
    }


    public static function set($name, $val)
    {
        $instance = self::getInstance();

        if (!isset($_SESSION['requested']))
            $_SESSION['requested'] = [];

        $_SESSION['requested']['set'] = (isset($_SESSION['requested']['set'])) ? $_SESSION['requested']['set'] + 1 : $_SESSION['requested']['set'] = 1;

        $instance->vars[$name] = $val;
        self::_updateSession();
    }


    public static function get($name)
    {
        $instance = self::getInstance();

        if (isset($_SESSION['requested']))
            $_SESSION['requested'] = [];

        if (isset($instance->vars[$name]))
        {
            $_SESSION['requested']['get'] = (isset($_SESSION['requested']['get'])) ? $_SESSION['requested']['get'] + 1 : $_SESSION['requested']['get'] = 1;
            return $instance->vars[$name];
        }
        $_SESSION['requested']['failed'] = (isset($_SESSION['requested']['failed'])) ? $_SESSION['requested']['failed'] + 1 : $_SESSION['requested']['failed'] = 1;
        $_SESSION['requested']['failed_vars'][] = $name;
    }

    public static function getVars()
    {
        if (self::getToken() &&
            isset($_SESSION[self::getSessionId()]['vars']))
        {
            return $_SESSION[self::getSessionId()]['vars'];
        }

        return array();
    }


    /**
     * Returns user API token or false if it's not set or expired
     *
     * @return mixed
     */
    public static function getToken()
    {
        if (isset($_SESSION[self::getSessionId()]) &&
            $_SESSION[self::getSessionId()]['token'] &&
            $_SESSION[self::getSessionId()]['expires'] &&
            $_SESSION[self::getSessionId()]['expires'] > time())
        {
            return $_SESSION[self::getSessionId()]['token'];
        }
        else
        {
            self::_deleteSession();
        }
    }


    /**
     * Returns user API token expiration timestamp or false if token is expired or not set
     *
     * @return mixed
     */
    public static function getTokenExpires()
    {
        if (self::getToken())
        {
            return $_SESSION[self::getSessionId()]['expires'];
        }
    }


    /**
     * Saves user data in a session
     *
     * @param $user
     */
    public static function setUser($user)
    {
        $instance = self::getInstance();
        $instance->user = $user;

        self::_updateSession();
    }


    /**
     * Returns user data from a session
     *
     * @return mixed
     */
    public static function getUser()
    {
        if (self::getToken() && isset($_SESSION[self::getSessionId()]) &&
            isset($_SESSION[self::getSessionId()]['user']))
        {
            return $_SESSION[self::getSessionId()]['user'];
        }
    }


    public static function destroy()
    {
        self::_deleteSession();
    }


    /**
     * Updates user session with values of the class instance
     */
    private static function _updateSession()
    {
        $instance = self::getInstance();

        $_SESSION[self::getSessionId()]['token']      = $instance->token;
        $_SESSION[self::getSessionId()]['expires']    = $instance->tokenExpires;
        $_SESSION[self::getSessionId()]['user']       = $instance->user;
        $_SESSION[self::getSessionId()]['vars']       = $instance->vars;
    }


    /**
     * Deletes user session
     */
    private static function _deleteSession()
    {
        if (isset($_SESSION))
        {
            unset($_SESSION[self::getSessionId()]);
        }
    }
} 