<?php

namespace hji\common\models;

class ServerDataCache
{

    const CACHE_PREFIX = 'hji_server_';

    /**
     * Default cache expiration - One day
     * @var int
     */
    public  $defaultExpiration = 86400;

    public function getData($name)
    {
        return get_transient(self::CACHE_PREFIX  . $name);
    }

    public function setData($name, $value, $expiration = null)
    {
        if ($expiration === null) {
            $expiration = $this->defaultExpiration;
        }

        return set_transient(self::CACHE_PREFIX . $name, $value, $expiration);
    }

    /**
     * @param $name
     * @return mixed
     */
    public function getCompressedData($name)
    {
        $compressedDecodedValue = get_transient(self::CACHE_PREFIX  . $name);
        if (!is_string($compressedDecodedValue)) {
            return false;
        }
        $compressedValue = base64_decode($compressedDecodedValue);
        if ($compressedValue === false || $compressedValue === null) {
            return false;
        }
        $serializedValue = gzuncompress($compressedValue);
        $value = unserialize($serializedValue);

        return $value;
    }

    /**
     * @param $name
     * @param $value
     * @param null $expiration
     * @return bool
     */
    public function setCompressedData($name, $value, $expiration = null)
    {
        if ($expiration === null) {
            $expiration = $this->defaultExpiration;
        }
        $compressedValue = base64_encode(gzcompress(serialize($value)));

        return set_transient(self::CACHE_PREFIX . $name, $compressedValue, $expiration);
    }

    /**
     * Use to receive object, saved by setSplitData
     * @param $name
     * @return bool|\stdClass
     */
    public function getSplitData($name)
    {
        $keys = get_transient(self::CACHE_PREFIX  . $name);
        if (empty($keys)) {
            return false;
        }

        $resultObject = new \stdClass();
        foreach ($keys as $key) {
            if (!is_string($key)) {
                return false;
            }
            $resultObject->{$key} = get_transient(self::CACHE_PREFIX  . $name . '_' . $key);
        }

        return $resultObject;
    }

    /**
     * Use in case if we have a big (more then 1 Mb) object, and we split it by his first level properties
     * @param $name
     * @param $value
     * @param null $expiration
     * @return bool
     */
    public function setSplitData($name, $value, $expiration = null)
    {
        if (is_string($value) || count((array)$value) < 1) {
            return false;
        }
        if ($expiration === null) {
            $expiration = $this->defaultExpiration;
        }
        $resultKeys = [];
        foreach ($value as $key => $item) {
            $resultKeys[] = $key;
            set_transient(self::CACHE_PREFIX . $name . '_' . $key, $item, $expiration);
        }

        return set_transient(self::CACHE_PREFIX . $name, $resultKeys, $expiration);
    }

    /**
     * Unset data for list or single data set
     * @param $names
     */
    public function unsetData($names)
    {
        if (!is_array($names)) {
            $names = array($names);
        }
        foreach ($names as $name) {
            delete_transient(self::CACHE_PREFIX  . $name);
        }
    }

 }