

hji.define(['require', 'jquery', 'underscore', 'hji-common/gMapsApi', 'hji-common/gMap', 'hji-common/_lookupArea'], function (require, $, _, GoogleMapsLoader, gMap) {
  var getArea = function getArea(options, callback) {
    var required = {
      // Area MLS market
      market: null,
      // Area Lookup Type
      lookupType: 'polygon',
      // Input field selector - user will enter area to lookup polygon
      lookupField: false
    };
    var optional = {
      // Target wrapper for HTML markup if  custom one is not provided
      target: false,
      // Preview area on the map (bool)
      mapPreview: true,
      // Drawing manager for the map
      drawingManager: true,
      // Container where map will be displayed
      mapCanvas: false,
      // Callback to pass area data to
      callback: typeof callback != 'undefined' ? callback : false
    }; // Component config

    this.config = $.extend({}, required, optional, options); // Google Map object

    this.map = false; // Selectize Obj - lookup field

    this.selectizeObj = false;
    this.setupUI(); // Check required options before doing anything

    var configFailed = false;

    for (var i in required) {
      if (!this.config.hasOwnProperty(i) || this.config[i] === null) {
        this.error('getArea component failed: ' + i + ' is a required option.');
        configFailed = true;
      }
    }

    if (configFailed) return false; // Initialize area lookup

    this.initAreaLookup();
    this.returnShapeCords();
  };

  getArea.prototype.setupUI = function () {
    var markup; // If target is set, then setup UI dynamically

    if (!this.config.target || !this.config.market) {
      return false;
    }

    markup = $('<div></div>').addClass('get-area-wrap hjitb');
    $('<div></div>').addClass('area-lookup-wrap').appendTo(markup);
    $('<input class="area-lookup type="text" name="polygon" data-market="' + this.config.market + '" placeholder="Neighborhood, city or zip">').appendTo(markup.find('.area-lookup-wrap'));
    $('<div></div>').addClass('location-preview').appendTo(markup);
    $(this.config.target).html(markup);
    this.config.lookupField = markup.find('.area-lookup')[0];
    this.config.mapCanvas = markup.find('.location-preview')[0];
  };

  getArea.prototype.initAreaLookup = function () {
    var that = this;
    var lookupOptions = {
      type: this.config.lookupType,
      market: this.config.market,
      getArea: true,
      callback: function callback(result) {
        if (that.config.mapPreview) {
          that.mapPreview(result);
        }
      }
    };

    require(['hji-common/_lookupArea'], function () {
      var $select = $(that.config.lookupField).lookupArea(lookupOptions);
      that.selectizeObj = $select[0].selectize;
      that.setListeners();
    }); // Setup default map if market centroid is known


    var m = window.HJI.vars.common.markets;

    if (m[this.config.market].centroid) {
      this.mapPreview(m[this.config.market]);
    }
  };

  getArea.prototype.setListeners = function () {
    var that = this; // On item removal

    this.selectizeObj.on('item_remove', function (value) {
      // Remove $ endpoint from the Polygon ID, since it was
      // prefixed for IDX searches.
      var id = value.replace('$', ''); // Search through shapes to see if current ID matches
      // to any shape ID on the map.

      if (that.map.shapes.length) {
        _.each(that.map.shapes, function (v, k, list) {
          if (v.hjiPolygonObj && v.hjiPolygonObj.id == id) {
            that.map.removeShape(v);
          }
        });
      }
    });
  };

  getArea.prototype.mapPreview = function (result) {
    var that = this; // Make sure map is loaded before calling any gMap methods

    GoogleMapsLoader.done(function () {
      if (!that.map) {
        var mapSettings = {
          canvas: $(that.config.mapCanvas)[0],
          drawingManager: that.config.drawingManager
        };
        var mapOptions = {
          center: new google.maps.LatLng(result.centroid.latitude, result.centroid.longitude),
          zoom: 9
        };
        that.map = new gMap(mapSettings, mapOptions, function () {
          that.mapPreview(result);
        });
      } else {
        if (typeof result.polygon == 'undefined') {
          return false;
        }

        if (result.polygon && result.polygon.length) {
          var coords = that.hjiPolygonToLatLng(result.polygon[0]);
          var polygonOptions = {
            editable: false,
            clickable: true,
            collectable: true,
            hjiPolygonObj: result,
            hjiPolygon: true
          };
          that.map.renderPolygon(coords, polygonOptions);
        }

        that.map.fitBounds();
      }
    });
    $(this.config.mapCanvas).css({
      width: '100%',
      height: '400px'
    });
    $(this.config.mapCanvas).on('shapeDelete', function (e, shape) {
      if (typeof shape.hjiPolygonObj != 'undefined') {
        that.selectizeObj.removeItem('$' + shape.hjiPolygonObj.id);
      }
    });
  };

  getArea.prototype.returnShapeCords = function () {
    var that = this;
    $(this.config.mapCanvas).on('shapesUpdated', function (event) {
      var data = [];

      _.each(that.map.shapes, function (shape, i, list) {
        if (shape.type == 'rectangle') {
          data.push(that.map.getRectangle(shape));
        } else if (shape.type == 'polygon') {
          data.push(that.map.getPolygon(shape));
        }
      });

      if (that.config.callback) {
        that.config.callback(that.toArea(data), data);
      }
    });
  };

  getArea.prototype.hjiPolygonToLatLng = function (data) {
    var coords = [];

    for (var i in data) {
      coords.push(this.map.strToLatLng(data[i]));
    }

    return coords;
  };

  getArea.prototype.error = function (message) {
    if (typeof console != 'undefined' && typeof console.error != 'undefined') {
      console.error(message);
    } else {
      throw new Error(message);
    }
  };
  /**
   * Converts polygon go GeoJSON Polygon
   * @param coords with lat/lng objects
   */


  getArea.prototype.toGeoJSON = function (polygons) {
    var result = [];

    _.each(polygons, function (polygon, k, list) {
      var coords = [];

      _.each(polygon, function (coord, i, l) {
        coords.push([coord.lng, coord.lat]);
      });

      result.push([coords]);
    });

    if (result.length > 1) {
      return {
        type: 'MultiPolygon',
        coordinates: result
      };
    } else if (result.length) {
      return {
        type: 'Polygon',
        coordinates: result[0]
      };
    }
  };

  getArea.prototype.toArea = function (data) {
    var hjiPolygons = [],
        // HJI polygons with .hjiData objects
    polygonIDs = [],
        // HJI polygon IDs
    customPolygons = [],
        // Custom drawn polygons
    geoJSON = null,
        // Custom polygons converted to GeoJSON.
    area = null; // Custom area result ready to be passed to the Slipstream API

    if (data.length) {
      _.each(data, function (v, k, list) {
        // If array item has an ID - it's an HJI polygon
        if (v.id) {
          // Multiple HJI polygons can have the same ID since they are part
          // of a single Multipolygon
          // Make sure we only grab ID and polygon's HJI object once
          if (_.indexOf(polygonIDs, v.id) == -1) {
            polygonIDs.push('$' + v.id);
            hjiPolygons.push(v.hjiData);
          }
        } // Otherwise it's a custom-drawn polygon
        else if (v.coords) {
            customPolygons.push(v.coords);
          }
      }); // Start adding polygon IDs and geoJSON into the area array.


      area = polygonIDs;
      geoJSON = this.toGeoJSON(customPolygons); // If geoJSON is not empty, add it into the area array

      if (geoJSON) {
        area.push(geoJSON);
      }

      if (area) {
        return area;
      }
    }

    return null;
  };

  return getArea;
});