

hji.define(['require', 'jquery', 'underscore', 'backbone', './contentModelFactory', './gfContentModel', './inlineModalView', './alerts', './modals'], function (require, $, _, Backbone, ContentModelFactory, GFContentModel) {
  var gfm = {};
  /**
   * Renders Gravity Form content in a Modal
   *
   * @param gform $jquery object of the gform content
   * @param target original target selector where gform was embedded
   */

  gfm.createGFView = function () {
    var InlineModalView = require('./inlineModalView');

    var ModalView = InlineModalView.extend({
      okClicked: function okClicked(modal) {
        var that = this; // Removes submit event added in GF v.2.4.0 that aborts
        // form submission when Submit button is hidden.

        $(document).off('submit.gravityforms', '.gform_wrapper form');
        this.$el.trigger('onSubmit'); // Ajax-enabled gravity form updates form content after submission
        // Therefore we set event listeners for the "after submission"
        // Delay to process submitted outcomes
        // so the content reattaches to the DOM

        setTimeout(function () {
          var form = $(that.model.get('selector')).find('form');

          if (form.length) {
            // Fire up an "progress modal", so user knows what's happening
            var alerts = require('./alerts');

            var progressModal = alerts.progress('Request is being processed...'); // Submit the re-attached form

            form.submit(); // Set events for an ajax-enabled gravity form, since it updates its content.

            var eventData = {
              gForm: that.model,
              progressModal: progressModal
            };
            that.trigger('onError', eventData);
            that.trigger('onSuccess', eventData);
          }
        }, 25);
      },
      onError: function onError(eventData) {
        gfm.Events().onValidation(eventData);
      },
      onSuccess: function onSuccess(eventData) {
        var that = this;
        gfm.Events().onConfirmation(eventData, function () {
          that.reattachOriginal();
        });
      }
    });
    return ModalView;
  };
  /**
   * Renders gravity form confirmation message in a modal
   *
   * @param target gravity form selector from where confirmation message is being retrieved
   * @returns {*} modal object
   */


  gfm.confirmationModal = function (target) {
    var confirmation = '<div class="alert alert-success">' + $(target).find('.gform_confirmation_message').html() + '</div>';

    var alerts = require('./alerts');

    return alerts.alert({
      content: confirmation,
      autoclose: true
    });
  };
  /**
   * Create modals on document ready
   * if content has to be displayed as soon as page loads.
   *
   * Note: only 1 on load event per TARGET - there are can be
   * multiple triggers for each target though.
   */


  gfm.setOnLoadEvents = function (targets) {
    $(document).ready(function (e) {
      _.each(targets, function (target, i, list) {
        // Check if form needs to be displayed on document ready
        var gForm = new GFContentModel({
          selector: target
        }); // Is validation message being displayed?

        if (gForm.isValidation()) {
          var modals = require('./modals');

          modals.createModal(target);
        } // Is confirmation message being displayed?
        else if (gForm.isConfirmation()) {
            gfm.confirmationModal(gForm.get('selector'));
          }
      });
    });
  };
  /**
   * Gravity Forms events
   *
   * @returns {{onValidation: Function, onConfirmation: Function}}
   * @constructor
   */


  gfm.Events = function () {
    return {
      onValidation: function onValidation(eventData, callback) {
        $(document).off('gform_page_loaded.gfModal');
        $(document).on('gform_page_loaded.gfModal', eventData, function (e, form_id, current_page) {
          // Check if current event is for our modal form
          if (!gfm.isFormEventValid(e, form_id)) {
            return false;
          } // If progress modal is rendered, close it


          if (e.data.progressModal && e.data.progressModal.isRendered) {
            e.data.progressModal.close();
          } // Call callback/display validation modal


          setTimeout(function () {
            gfm.validationEventModal(e.data.gForm);

            if (callback) {
              callback(e, form_id, current_page);
            }
          }, 25);
        });
      },
      onConfirmation: function onConfirmation(eventData, callback) {
        $(document).off('gform_confirmation_loaded.fgModal');
        $(document).on('gform_confirmation_loaded.fgModal', eventData, function (e, form_id) {
          // Check if current event is for our modal form
          if (!gfm.isFormEventValid(e, form_id)) {
            return false;
          } // If progress modal is rendered, close it


          if (e.data.progressModal && e.data.progressModal.isRendered) {
            e.data.progressModal.close();
          }

          gfm.confirmationModal(e.data.gForm.get('selector'));

          if (callback) {
            callback(e, form_id);
          }
        });
      }
    };
  };
  /**
   * Determines if current event is for intended modal
   *
   * @param event     event object expected to contain gForm Model
   * @param form_id   gravity form ID passed by the gravity form event
   * @returns {boolean}
   */


  gfm.isFormEventValid = function (event, form_id) {
    // Does gForm data exist
    if (!event.data.gForm) {
      return false;
    } // Is gForm data == form_id passed by the event


    if (event.data.gForm.get('form').id == form_id) {
      return true;
    }

    return false;
  };
  /**
   * Renders Gravity Form with validation errors in a Modal
   *
   * @param gForm
   */


  gfm.validationEventModal = function (contentModel) {
    var modals = require('./modals');

    modals.createModal(contentModel);
  };

  return gfm;
});