

hji.define(['require', 'jquery', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, Widget) {
  /**
   * All Area Widgets
   *
   * Parameters:
   *      @param data-init string         widgets/area
   *      @param data-id string           Area ID
   *      @param data-chooser boolean     Render Chooser true|false
   *      @param data-chooser-limit       number of items to be returned within the chooser
   *      @param data-widgets JSON Array  '["age", "cost", "education", "employment",
   *                                        "occupancy", "summary", "weather", "details"]'
   *
   *  Example:
   *      <div
   *          data-init="widgets/area"
   *          data-chooser="true"
   *          data-id="6e87e3e7bf04cda3ce8bdaecf473e2d8" // Paradise Valley AZ
   *          data-widgets='["age", "cost", "education", "employment", "occupancy", "summary", "weather", "details"]'
   *          >
   *      </div>
   *
   *      Widget Object can be accessed in the DOM as follows:
   *          $('[data-init="widgets/area"]')[0].widget;
   *
   *  Manipulating widget instance:
   *      // Re-setting widgets to only summary
   *      $('[data-init="widgets/area"]')[0].widget.options.widgets = ['summary'];
   *      // Setting new area ID
   *      $('[data-init="widgets/area"]')[0].widget.options.id = 'ac28dd4fdde34d4040931bca6459ceea';
   *      // Re-rendering the whole widget
   *      $('[data-init="widgets/area"]')[0].render();
   */
  var areaWidget = function areaWidget() {
    this.options = {
      widgets: ['details']
    };
    this.loaded = [];
  };

  areaWidget.prototype = new Widget();

  areaWidget.prototype.render = function () {
    // Reset object instance, so it can be easily re-rendered with set options.
    this.reset();
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/areas'], function () {
        if (that.options.chooser) {
          that.loadChooser();
        }

        if (that.options.id) {
          that.loadWidgets(that.options.id);
        }
      });
    });
  };

  areaWidget.prototype.loadWidgets = function (areaId) {
    var that = this;
    that.loadArea(areaId, function () {
      if (that.loaded.length) return;

      for (var i in that.options.widgets) {
        var widget = that.options.widgets[i];
        var widgetWrapper = that.$el.find('.widget-wrapper.' + widget + '-widget');

        if (!widgetWrapper.length) {
          widgetWrapper = $('<div class="widget-wrapper ' + widget + '-widget"></div>');
          that.$el.first('.slipstream-areas').append(widgetWrapper);
        }

        that.loadWidget(widget);
      }
    });
  };

  areaWidget.prototype.loadWidget = function (widget) {
    var that = this;

    window.require(['slipstream/areas/widget/' + widget], function (Widget) {
      /*
       * If we have not already done so, create the /areas/widget/details
       * widget to display the model.
       *
       * https://slipstream.homejunction.com/v1/docs/js/areas/widget/details
       */
      if (!that.loaded[widget]) {
        that.loaded[widget] = Widget.init({
          renderTo: that.$el.find('.widget-wrapper.' + widget + '-widget'),
          model: that.area
        });
      }
    });
  };

  areaWidget.prototype.loadChooser = function () {
    var that = this;

    require(['hji-common/widgets/chooser'], function (Chooser) {
      var options = {
        limit: that.options.chooserLimit
      };
      Chooser.init(that.$el.find('.chooser-wrapper'), options).onSelect(function (id) {
        that.loadWidgets(id);
      });
    });
  };

  areaWidget.prototype.loadArea = function (id, callback) {
    var that = this;

    window.require(['slipstream/areas/model/area'], function (Area) {
      /*
       * An /areas/model/area keeps track of the selected area.
       *
       * https://slipstream.homejunction.com/v1/docs/js/areas/model/area
       */
      if (!that.area) that.area = new Area();
      /*
       * Make sure we are not already displaying this area.    There is no
       * reason to refresh the page unless we absolutely have to.
       */

      var nowShowing = that.area.get('id');

      if (!id || nowShowing === id) {
        return; // Nothing to do!
      }
      /*
       * We now fetch the selected area.
       *
       * https://slipstream.homejunction.com/v1/docs/js/areas/model/area
       */


      that.area.fetch({
        parameters: {
          id: id,
          details: true,
          geometry: true
        }
      });
      that.area.on('change', function () {
        if (typeof callback === 'function') {
          callback();
        }
      });
    });
  };

  areaWidget.prototype.template = function () {
    var template = $('<div class="slipstream">').append($('<div class="slipstream-areas">'));
    if (this.options.chooser) template.find('.slipstream-areas').prepend('<div class="chooser-wrapper">');
    return template;
  };
  /**
   * Resets areaWidget instance so it can be re-rendered.
   */


  areaWidget.prototype.reset = function () {
    this.area = null;
    this.loaded = [];
  };

  return areaWidget;
});