

hji.define(['require', 'jquery', '_', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, _, Widget) {
  /**
   * Sales Catalogue Widget
   *
   * Renders list of sold properties and corresponding map representation with plotted markers.
   * @source https://slipstream.homejunction.com/docs/js/1.3.3/sales/widget/catalogue
   *
   * @todo extract lookBack() method into sales parameters validation utility for re-usability
   *
   * Invocation Example:
   *  <div data-init="widgets/sales/catalogue" data-parameters='{"city":"San Diego"}'></div>
   *
   *  Initialized component object can be accessed as follows:
   *   jQuery('[data-init="widgets/sales/catalogue"]')[0].widget;
   *
   * Supported data attributes:
   *
   * @param string data-init Initializes plugin by given namespace.
   * @param object data-parameters JSON Object of sales search parameters.
   *               Note: all Sales Search web service parameters are supported:
   *               @api https://slipstream.homejunction.com/docs/ws/sales#search
   *
   * Dynamic Parameters:
   *  saleDate supports lookBack() method that generates correct query for the last number of days
   *  Example:
   *  data-parameters='{"saleDate":"lookBack(60)"}' - will return sales within the last 60 days
   *
   */
  var salesWidget = function salesWidget() {
    this.options = {
      init: null,
      // component name
      parameters: {} // widget parameters

    };
    this.loaded = [];
  };

  salesWidget.prototype = new Widget();

  salesWidget.prototype.render = function () {
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/sales'], function () {
        if (_.has(that.options, 'parameters')) {
          that.loadWidget(that.options.parameters);
        }
      });
    });
  };

  salesWidget.prototype.loadWidget = function (parameters) {
    if (this.loaded.length) return;
    var that = this;
    var widgetWrapper = that.$el.find('.widget-wrapper.sales-widget');

    if (!widgetWrapper.length) {
      widgetWrapper = $('<div class="widget-wrapper sales-widget"></div>');
      that.$el.append(widgetWrapper);
    }

    window.require(['slipstream/sales/model/sales', 'slipstream/sales/widget/catalogue'], function (Sales, Catalogue) {
      // Compute the date range between 6 months ago and today
      var today = new Date();
      var then = new Date();
      then.setMonth(today.getMonth() - 6); // 6 months ago

      var range = [then.getMonth() + 1, then.getDate(), then.getFullYear()].join('/') + ':' + [today.getMonth() + 1, today.getDate(), today.getFullYear()].join('/'); // Fetch the sales

      var sales = new Sales();
      var defaultParams = {
        saleDate: range,
        sortField: 'saleDate',
        sortOrder: 'DESC',
        limit: 150
      };
      var params = that.processParams(_.defaults(parameters, defaultParams));
      sales.fetch({
        parameters: params
      }); // Display the widget

      that.loaded['catalogue'] = Catalogue.init({
        renderTo: that.$el.find('.widget-wrapper'),
        model: sales
      });
    });
  };

  salesWidget.prototype.processParams = function (params) {
    if (_.has(params, 'saleDate')) {
      // Is a lookBack?
      var pattern = /lookBack/g;

      if (pattern.test(params.saleDate)) {
        var result = params.saleDate.match(/\d+/g);
        var days = parseInt(result[0]);

        if (days) {
          var date = new Date();
          date.setDate(date.getDate() - days);
          params.saleDate = '>' + Math.round(date.getTime() / 1000);
        }
      }
    }

    return params;
  };

  return salesWidget;
});