

hji.define(['require', 'jquery', 'underscore', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, _, Widget) {
  /**
   * Schools Widget
   *
   * Parameters:
   *      @param data-init string         widgets/school
   *      @param data-parameters JSON Object  supports all schools filters
   *
   * Filters (data-parameters):
   *      admission: Public | Private
   *      charter: true | false
   *      district: Name |ID
   *      grade: elementary, middle, high, pk, kg, 1, 2, 3, etc.
   *      magnet: true
   *      name: name of the school
   *      titlei: true to limit the search to only Title I schools.
   *
   * Note: adding area selector (data-chooser) to the front-end,
   *       will reset all pre-set filters/parameters just to the selected area/polygon.
   *
   *
   * Example 1:
   *      Renders table of Magnet schools in Phoenix, AZ
   *      <div data-init="widgets/schools" data-parameters='{"city":"Phoenix", "magnet":"true"}'></div>
   *
   * Example 2:
   *      Renders table of high table in the Paradise Valley Unified District
   *      <div data-init="widgets/schools" data-parameters='{"district":"Paradise Valley Unified", "grade":"high"}' data-chooser="true"></div>
   *
   *      Widget Object can be accessed in the DOM as follows:
   *      var widget = $('[data-init="widgets/schools"]')[0].widget;
   *
   * Manipulating widget instance:
   *      var widget = $('[data-init="widgets/schools"]')[0].widget;
   *      // Resetting search parameters to a polygon
   *      widget.options.parameters = {'polygon' : '$ac28dd4fdde34d4040931bca6459ceea'};
   *      // Re-rendering the whole widget
   *      widget.render();
   */
  var schoolsWidget = function schoolsWidget() {
    this.$el = null;
    this.params = {};
    this.widgets = ['table'];
    this.options = {};
    this.loaded = [];
    this.model = null;
  };

  schoolsWidget.prototype = new Widget();

  schoolsWidget.prototype.init = function ($el, options) {
    this.$el = $el;
    this.$el[0][this.moduleName] = this;
    if (_.has(options, 'widgets')) this.widgets = options.widgets;
    $.extend(this.options, _.omit(options, 'widgets'));
    this.render();
  };

  schoolsWidget.prototype.render = function () {
    // Reset object instance, so it can be easily re-rendered with set options.
    this.reset();
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/schools'], function () {
        if (that.options.chooser) {
          that.loadChooser();
        }

        that.loadWidgets();
      });
    });
  };

  schoolsWidget.prototype.loadWidgets = function (polygon) {
    for (var i in this.widgets) {
      var widget = this.widgets[i];
      var widgetWrapper = this.$el.find('.widget-wrapper.' + widget + '-widget');

      if (!widgetWrapper.length) {
        widgetWrapper = $('<div class="widget-wrapper ' + widget + '-widget"></div>');
        this.$el.first('.slipstream-areas').append(widgetWrapper);
      }

      if (polygon) {
        this.options.parameters = {
          polygon: '$' + polygon
        };
      }

      this.loadWidget(widget);
    }
  };

  schoolsWidget.prototype.loadWidget = function (widget) {
    var that = this;

    window.require(['slipstream/schools/model/schools', 'slipstream/schools/widget/' + widget], function (Model, Widget) {
      if (!that.model) that.model = new Model();
      var target = that.$el.find('.widget-wrapper.' + widget + '-widget'),
          viewOptions = {
        renderTo: target,
        model: that.model
      };
      that.updateModel(); // If widget is not rendered, render it, otherwise,
      // change of model will change the view automatically

      if (!that.loaded[widget]) {
        that.loaded[widget] = Widget.init(viewOptions);
      }
    });
  };

  schoolsWidget.prototype.updateModel = function () {
    if (!this.model) console.error('Model is not available. Widget has not been initialized.');
    this.model.fetch({
      parameters: this.options.parameters
    });
  };

  schoolsWidget.prototype.loadChooser = function () {
    var that = this;

    require(['hji-common/widgets/chooser'], function (Chooser) {
      var options = {
        limit: that.options.chooserLimit
      };
      Chooser.init(that.$el.find('.chooser-wrapper'), options).onSelect(function (polygon) {
        that.loadWidgets(polygon);
      });
    });
  };

  schoolsWidget.prototype.template = function () {
    var template = $('<div class="slipstream">').append($('<div class="slipstream-school">').append($('<div class="details-wrapper">')));
    var template = $('<div class="slipstream">').append($('<div class="slipstream-school">'));
    if (this.options.chooser) template.find('.slipstream-school').prepend('<div class="chooser-wrapper">');
    return template;
  };
  /**
   * Resets widget instance so it can be re-rendered.
   */


  schoolsWidget.prototype.reset = function () {
    this.model = null;
    this.loaded = [];
  };

  return schoolsWidget;
});